@@ -15,13 +15,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *----------------------------------------------------------------------------
+ * Version 2.07
+ *   - Fix two bugs in timecode parsing function
+ *   - Move most inline assembly to yasm
+ *
  * Version 2.06
  *   - Add option to add an offset to output timecodes
  *   - Change return value when no events were detected to 0
  *   - Add option to allow writing of empty output files
  *   - Add option to enforce even Y coordinates (may help avoid problems with
  *     interlaced video in DVD related use cases)
- *   - Fix SSE2 with more recent GCC version
+ *   - Fix SSE2 with more recent GCC versions
  *   - Add option for stricter checks on buffer/palette limits for SUPs
  *   - Fix bug where LastEventOutTC could be one frame early
  *   - Add additional checks to commandline argument parsing
@@ -438,142 +442,10 @@ void write_png(char *dir, int file_id, uint8_t *image, int w, int h, int graphic
 	fclose(fh);
 }
 
-int is_identical_sse2 (stream_info_t *s_info, char volatile *img, char volatile *img_old)
-{
-	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
-	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
-	char volatile full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
-	int volatile different = 0;
-	int volatile *diff_ptr = &different;
-
-	asm __volatile__ ("movupd (%0), %%xmm3\n" : : "r" (zero));
-	asm __volatile__ ("movupd (%0), %%xmm4\n" : : "r" (a_mask));
-	asm __volatile__ ("movupd (%0), %%xmm7\n" : : "r" (full));
-	while (img < max && !different)
-	{
-		asm __volatile__
-		(
-			"movapd (%%eax), %%xmm0\n"
-			"movapd %%xmm0, %%xmm5\n"
-			"pand %%xmm4, %%xmm5\n"
-			"movapd (%%esi), %%xmm1\n"
-			"pcmpeqd %%xmm3, %%xmm5\n"
-			"pxor %%xmm7, %%xmm5\n"
-			"pand %%xmm5, %%xmm0\n"
-			"movapd %%xmm0, (%%eax)\n"
-			"psadbw %%xmm1, %%xmm0\n"
-			"movd %%xmm0, (%%edx)\n"
-			: "=a" (img), "=d" (diff_ptr)
-			: "a" (img), "S" (img_old), "d" (diff_ptr)
-			: "memory"
-		);
-		img += 16;
-		img_old += 16;
-	}
-
-	if (different)
-		return 0;
-
-	return 1;
-}
-
-int is_empty_sse2 (stream_info_t *s_info, char volatile *img)
-{
-	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
-	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
-	int volatile not_empty = 0;
-	int volatile *ne_ptr = &not_empty;
-
-	asm __volatile__ ("movupd (%0), %%xmm1\n" : : "r" (zero));
-	asm __volatile__ ("movupd (%0), %%xmm2\n" : : "r" (a_mask));
-	while (img < max && !not_empty)
-	{
-		asm __volatile__
-		(
-			"movapd (%%esi), %%xmm0\n"
-			"pand %%xmm2, %%xmm0\n"
-			"psadbw %%xmm1, %%xmm0\n"
-			"movd %%xmm0, (%%edx)\n"
-			: "=d" (ne_ptr)
-			: "S" (img), "d" (ne_ptr)
-			: "memory"
-		);
-		img += 16;
-	}
-
-	if (not_empty)
-		return 0;
-
-	return 1;
-}
-
-void zero_transparent_sse2 (stream_info_t *s_info, char volatile *img)
-{
-	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
-	char volatile zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-	char volatile a_mask[16] = {0,0,0,0xff,0,0,0,0xff,0,0,0,0xff,0,0,0,0xff};
-	char volatile full[16] = {0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};
-
-	asm __volatile__ ("movupd (%0), %%xmm1\n" : : "r" (zero));
-	asm __volatile__ ("movupd (%0), %%xmm2\n" : : "r" (a_mask));
-	asm __volatile__ ("movupd (%0), %%xmm3\n" : : "r" (full));
-	while (img < max)
-	{
-		asm __volatile__
-		(
-			"movapd (%%eax), %%xmm0\n"
-			"movapd %%xmm0, %%xmm5\n"
-			"pand %%xmm2, %%xmm5\n"
-			"pcmpeqd %%xmm1, %%xmm5\n"
-			"pxor %%xmm3, %%xmm5\n"
-			"pand %%xmm5, %%xmm0\n"
-			"movapd %%xmm0, (%%eax)\n"
-			: "=a" (img)
-			: "a" (img)
-		);
-		img += 16;
-	}
-}
-
-void swap_rb_sse2 (stream_info_t *s_info, char volatile *img, char volatile *out)
-{
-	char volatile *max = img + s_info->i_width * s_info->i_height * 4;
-	char volatile ag_mask[16] = {0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff};
-	char volatile rb_mask[16] = {0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0,0xff,0};
-
-	/* Set up masks */
-	asm volatile
-	(
-		"movupd (%0), %%xmm0\n"
-		"movupd (%1), %%xmm5\n"
-		:
-		: "r" (ag_mask), "r" (rb_mask)
-	);
-
-	/* Convert */
-	while (img < max)
-	{
-		asm volatile
-		(
-			"movdqa (%%esi), %%xmm1\n"
-			"movdqa %%xmm1, %%xmm2\n"
-			"movdqa %%xmm1, %%xmm7\n"
-			"psrld $16, %%xmm2\n"
-			"pslld $16, %%xmm1\n"
-			"por %%xmm2, %%xmm1\n"
-			"pand %%xmm0, %%xmm7\n"
-			"pand %%xmm5, %%xmm1\n"
-			"por %%xmm7, %%xmm1\n"
-			"movdqa %%xmm1, (%%edi)\n"
-			: "=D" (out)
-			: "S" (img), "D" (out)
-		);
-		img += 16;
-		out += 16;
-	}
-}
+extern int asm_is_identical_sse2 (stream_info_t *s_info, char *img, char *img_old);
+extern int asm_is_empty_sse2 (stream_info_t *s_info, char *img);
+extern void asm_zero_transparent_sse2 (stream_info_t *s_info, char volatile *img);
+extern void asm_swap_rb_sse2 (stream_info_t *s_info, char volatile *img, char volatile *out);
 
 int is_identical_c (stream_info_t *s_info, char *img, char *img_old)
 {
@@ -641,12 +513,6 @@ int detect_sse2 ()
 	unsigned int func = 0x00000001;
 	unsigned int eax, ebx, ecx, edx;
 
-	/* FIXME: SSE2 broken with new GCC. */
-/*#if __GNUC__ != 3
-#warning "Disabling SSE2 assembly."
-	return 0;
-#endif*/
-
 	if (detection != -1)
 		return detection;
 
@@ -673,7 +539,7 @@ int detect_sse2 ()
 int is_identical (stream_info_t *s_info, char *img, char *img_old)
 {
 	if (detect_sse2())
-		return is_identical_sse2(s_info, img, img_old);
+		return asm_is_identical_sse2(s_info, img, img_old);
 	else
 		return is_identical_c(s_info, img, img_old);
 }
@@ -681,7 +547,7 @@ int is_identical (stream_info_t *s_info, char *img, char *img_old)
 int is_empty (stream_info_t *s_info, char *img)
 {
 	if (detect_sse2())
-		return is_empty_sse2(s_info, img);
+		return asm_is_empty_sse2(s_info, img);
 	else
 		return is_empty_c(s_info, img);
 }
@@ -689,7 +555,7 @@ int is_empty (stream_info_t *s_info, char *img)
 void zero_transparent (stream_info_t *s_info, char *img)
 {
 	if (detect_sse2())
-		return zero_transparent_sse2(s_info, img);
+		return asm_zero_transparent_sse2(s_info, img);
 	else
 		return zero_transparent_c(s_info, img);
 }
@@ -697,7 +563,7 @@ void zero_transparent (stream_info_t *s_info, char *img)
 void swap_rb (stream_info_t *s_info, char *img, char *out)
 {
 	if (detect_sse2())
-		return swap_rb_sse2(s_info, img, out);
+		return asm_swap_rb_sse2(s_info, img, out);
 	else
 		return swap_rb_c(s_info, img, out);
 }
@@ -733,7 +599,7 @@ void mk_timecode (int frame, int fps, char *buf) /* buf must have length 12 (inc
 void print_usage ()
 {
 	fprintf(stderr,
-		"avs2bdnxml 2.06\n\n"
+		"avs2bdnxml 2.07\n\n"
 		"Usage: avs2bdnxml [options] -o output input\n\n"
 		"Input has to be an AviSynth script with RGBA as output colorspace\n\n"
 		"  -o, --output <string>        Output file in BDN XML format\n"
@@ -825,12 +691,12 @@ int parse_tc(char *in, int fps)
 	}
 	in[2] = 0;
 	in[5] = 0;
-	in[7] = 0;
+	in[8] = 0;
 	h = parse_int(in,     "t-offset hours",   NULL);
 	m = parse_int(in + 3, "t-offset minutes", NULL);
 	s = parse_int(in + 6, "t-offset seconds", NULL);
 	f = parse_int(in + 9, "t-offset frames",  NULL);
-	r += f;
+	r = f;
 	r += s * fps;
 	fps *= 60;
 	r += m * fps;
